# R code demonstrating a Bayesian analysis using R and JAGS

# ACED Example available from http://ecd.ralmond.net/ecdwiki/ACED/ACED

# The data are contained in 'ACED-Geometric.csv' 
#     available from http://ecd.ralmond.net/ACED/ACED-Geometric.csv
#     linked off the main data page http://ecd.ralmond.net/ecdwiki/ACED/Data

# To run this file:

# Install R, JAGS, and the following R packages:
#   rjags
#   mcmcplots
#   any packages they require or depend on

# Create a folder called 'ACED'
#   Currently, the code assumes this is in the C drive as C:\ACED
#   This can be modified below

# In the ACED folder, create a subfolder called 'Data'
# Store the 'ACED-Geometric.csv' file in the 'ACED\Data' folder

# In the ACED folder, create a subfolder called 'R'
# Store this file in the 'ACED\R' folder



rm(list=ls())


# Define the seed for reproducibility ---------------
set.seed(212814)


# Call the packages ------------
library(rjags)
library(mcmcplots)


# Define folders with locations ---------------
# Here is where you can change the location of the "main folder" fo ACED

main.folder <- "C:\\ACED\\"
data.folder <- paste0(main.folder, "Data\\")
R.folder <- paste0(main.folder, "R\\")
jags.folder <- paste0(main.folder, "jags\\")
dir.create(jags.folder)


# Read in the raw data ---------------
raw.data <- read.csv(paste0(data.folder, "ACED-Geometric.csv"), na.strings="NA", header = TRUE)


# Subset the data as desired ---------------
# Only including responses to Common Ratio tasks
common.ratio.data <- raw.data[ , 80:85]


# Define data used in the model -----------
x <- common.ratio.data-1  # convert to 1-0 data for dichotomous variable
n = nrow(x)               # sample size
J = ncol(x)               # number of observables
C = 2                     # number of categories (classes) of the latent variable


# Arrange the data to supply to JAGS -----------
jags.data <- list("x"=x, "n"=n, "J"=J, "C"=C)


# Define the model in JAGS code ----------------
modelstring <- as.character("

model{

###############################################################
# Conditional probability of the observables
# via a latent class model
###############################################################

for (i in 1:n){
  for(j in 1:J){
    x[i,j] ~ dbern(pi[theta[i],j])
  }
}


###############################################################
# Prior distribution for the latent variables
###############################################################
for(i in 1:n){
  theta[i] ~ dcat(lambda[])
}

###############################################################
# Prior distribution for the parameters
# that govern the distribution of the latent variables
###############################################################
lambda[1:C] ~ ddirch(alpha_lambda[])
for(c in 1:C){
  alpha_lambda[c] <- 1
}

###############################################################
# Prior distribution for the measurement model parameters
###############################################################
pi[1,1] ~ dbeta(1,1) 
pi[2,1] ~ dbeta(1,1) T(pi[1,1], )

for(c in 1:C){
  for(j in 2:J){
    pi[c,j] ~ dbeta(1,1)
  }
}


} # closes the model


") # closes the model as string



# Write out the code to a file
model.file.name <- "ACED model.txt"
write(x=modelstring, file=paste0(jags.folder, model.file.name), append=FALSE)


# Define parameters to monitor ----------
parameters <- c("lambda", "pi", "theta")


# Define initial values ---------------
lambda.inits.1 <- c(.9, .1)
pi.inits.1 <- NULL
for(j in 1:J){
  pi.inits.1 <- cbind(pi.inits.1, sort(runif(C, min=0, max=.5)))
}

lambda.inits.2 <- c(.1, .9)
pi.inits.2 <- NULL
for(j in 1:J){
  pi.inits.2 <- cbind(pi.inits.2, sort(runif(C, min=.5, max=1)))
}

lambda.inits.3 <- c(.5, .5)
pi.inits.3 <- NULL
for(j in 1:J){
  pi.inits.3 <- cbind(pi.inits.3, sort(runif(C, min=.25, max=.75)))
}


inits1 <- list(lambda=lambda.inits.1, pi=pi.inits.1)
inits2 <- list(lambda=lambda.inits.2, pi=pi.inits.2)
inits3 <- list(lambda=lambda.inits.3, pi=pi.inits.3)

inits <- list(inits1, inits2, inits3)


# Choose features of MCMC --------
#   the number of chains
#   the number of iterations to burn-in, 
#	  the number of iterations to thin by,
#	  the total number of iterations 
n.chains = 3
n.burnin = 0
n.thin = 1
n.iters.total.per.chain = 4000
n.iters.per.chain.after.burnin = n.iters.total.per.chain - n.burnin


# Run MCMC  --------

# Initialize the model
# Note that JAGS takes 1000 iterations to adapt the sampler
setwd(jags.folder)
jags.model.fitted <- jags.model(file=model.file.name,
                                  data=jags.data,                                       
                                  inits=inits,
                                  n.chains=n.chains
)

# Perform the burn-in iterations.
if(n.burnin>0) {
  update(jags.model.fitted,
           n.iter=n.burnin,
           variable.names=parameters,
           progress.bar="gui"
  )
}

# Now construct the posterior distribution.
# Recall that JAGS took 1000 iterations to adapt the sampler
jags.model.fitted <- coda.samples(
  jags.model.fitted,
  variable.names=parameters,
  n.iter=n.iters.per.chain.after.burnin,
  thin=n.thin, 
  progress.bar="gui"
)


# Convergence Assessment --------

# Create a subfolder for it
convergence.folder <- paste(jags.folder, "Convergence Assessment\\", sep="")
dir.create(convergence.folder)

# Move to it
setwd(convergence.folder)


# Define the draws from MCMC
draws.from.mcmc <- jags.model.fitted


# Define the parameters to use for convergence assessment
parameters.for.convergence <- colnames(as.matrix(draws.from.mcmc))
parameters.to.drop <- paste0("theta[", seq(1:n), "]")
parameters.for.convergence <- parameters.for.convergence[!is.element(parameters.for.convergence, parameters.to.drop)]


# Select the iterations to investigate
draws.to.analyze <- window(draws.from.mcmc,
                           start=n.burnin+1)


# Plot the results
mcmcplot(
  mcmcout = draws.to.analyze,
  parms = parameters.for.convergence,
  dir = getwd(),
  filename = "MCMC Plots for Convergence Assessment"
)

# Gelman-Rubin Diagnostics
gelman.diag(
  x=draws.to.analyze,
  multivariate=FALSE
)


# Posterior Summary --------

# Create a subfolder for it
posterior.summary.folder <- paste(jags.folder, "Summary of Posterior\\", sep="")
dir.create(posterior.summary.folder)

# Move to it
setwd(posterior.summary.folder)


# Decide on the burn-in 
n.burnin=3000


# Select the iterations to investigate
draws.to.analyze <- window(draws.from.mcmc,
                           start=n.burnin+1)


# Consider the effective sample size for estimating the mean of each parameter
effectiveSize(draws.to.analyze)


# Plot the results
mcmcplot(
  mcmcout = draws.to.analyze,
  dir = getwd(),
  filename = "MCMC Plots Iterations from Final Analysis"
)


# Combine chains for summaries
coda.options(combine.stats=TRUE, combine.plots=TRUE)


# Extract the summary statistics
#   Usual
#   Percentiles
#   HPD
summary.stats <- summary(draws.to.analyze)

# Combine chains for HPD
str(draws.to.analyze)
draws.to.analyze.as.one.list <- as.mcmc(do.call(rbind,draws.to.analyze)) 
str(draws.to.analyze.as.one.list) 


# HPDs for each parameter
medians <- apply(draws.to.analyze.as.one.list, 2, median)

# HPDs for each parameter
probability.for.HPD=.95
HPD.interval <- HPDinterval(draws.to.analyze.as.one.list, prob=probability.for.HPD)
# HPDinterval(draws.to.analyze, prob=probability.for.HPD) # uncomment if only using one chain



# Organize the summary statistics
summary.statistics <- cbind(
  summary.stats$statistics, 
  summary.stats$quantiles, 
  medians,
  matrix(HPD.interval, ncol=2)
)

colnames(summary.statistics) <- c(
  colnames(summary.stats$statistics),
  colnames(summary.stats$quantiles),
  c("Median", "95% HPD lower", "95% HPD Upper")
)

# Write out the summary statistics
write.csv(
  x=summary.statistics,
  file="Summary Statistics.csv"
)




